@@ -12,24 +12,22 @@ require DynaLoader;
 
 use vars qw/@ISA $VERSION/;
 @ISA = qw(Exporter DynaLoader);
-$VERSION = '1.13';
+$VERSION = '1.15';
 
 bootstrap Math::BigInt::GMP $VERSION;
 
 sub import { }		# catch and throw away
 
-##############################################################################
-# actual math code
+sub api_version() { 1; }	# we are compatible with MBI v1.70 and up
 
-sub _sub
+BEGIN
   {
-  # $x is always larger than $y! So overflow/underflow can not happen here
-  if ($_[3])
-    {
-    $_[2] = Math::BigInt::GMP::sub_two($_[1],$_[2]); return $_[2];
-    }
-  Math::BigInt::GMP::_sub_in_place($_[1],$_[2]);
-  }                                                                             
+  # both _num and _str just return a string
+  *_str = \&_num;
+  }
+
+##############################################################################
+# actual math code (this could also be put into GMP.xs for more speed)
 
 sub _div
   {
@@ -73,6 +71,57 @@ sub _check
   0;
   }
 
+sub _log_int
+  {
+  my ($c,$x,$base) = @_;
+
+  # X == 0 => NaN
+  return if _is_zero($c,$x);
+  # BASE 0 or 1 => NaN
+  return if _is_zero($c,$base) || _is_one($c,$base);
+
+  my $cmp = _acmp($c,$x,$base); 	# X == BASE => 1
+  if ($cmp == 0)
+    {
+    # return one
+    return (_one($c), 1);
+    }
+  # X < BASE
+  if ($cmp < 0)
+    {
+    return (_zero($c),undef);
+    }
+
+  my $trial = _copy($c,$base);
+  my $x_org = _copy($c,$x);
+  $x = _one($c);
+
+  my $a;
+  my $base_mul = _mul($c, _copy($c,$base), $base);
+  my $two = _two($c);
+
+  while (($a = _acmp($c, $trial, $x_org)) < 0)
+    {
+    _mul($c,$trial,$base_mul); _add($c, $x, $two);
+    }
+
+  my $exact = 1;
+  if ($a > 0)
+    {
+    # overstepped the result
+    _dec($c, $x);
+    _div($c,$trial,$base);
+    $a = _acmp($c,$trial,$x_org);
+    if ($a > 0)
+      {
+      _dec($c, $x);
+      }
+    $exact = 0 if $a != 0;
+    }
+
+  return ($x,$exact);
+  }
+
 1;
 __END__
 
@@ -98,7 +147,7 @@ the same terms as Perl itself.
 
 =head1 AUTHOR
 
-Tels <http://bloodgate.com/> in 2001-2003.
+Tels <http://bloodgate.com/> in 2001-2004.
 
 Thanx to Chip Turner for providing Math::GMP, which was inspiring my work.
 
